/* MT: Changed to Repeated Random version.  */

/*-------------------------------------------------------------------*/
/*                                                                   */
/* CBRAND.C       Juha Kivijrvi                                     */
/*                Marko Tuononen                                     */
/*                                                                   */
/*-------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <values.h>

#define  FACTFILE "cbrand.fac"
#include "parametr.c"
#include "memctrl.h"
#include "random.h"
#include "cb.h"
#include "file.h"
#include "owntypes.h"

/*--------------------------------------------------------------------*/


#define ProgName        "CBRAND"
#define VersionNumber   "version 0.3"
#define LastUpdated     "02.02.2006"        /* MT */


/*============================  P R I N T  ===============================*/


void PrintInfo(void)
{
  printf("\n%s %s          %s\n", ProgName, VersionNumber, LastUpdated);
  printf("Repeated Random - generates random codebooks and return best book found.\n");
  printf("Usage: %s [%coption] <TSfile> <CBfile>\n", ProgName, OPTION_SYMBOL);
  printf("\n  Options:\n");
  PrintOptions();
  printf("\n");
}


/*================  C O D E B O O K S  ==============================*/


int FreqToIndex(int freqsum, TRAININGSET* TS)

{
int i=0;
int l=0;

while (l<freqsum)
    {
    l += VectorFreq(TS,i);
    i++;
    }
return (i-1);
}


/*-------------------------------------------------------------------*/


void GetRandomVector(BOOKNODE* node, TRAININGSET* TS)
{
int t;

t=irand(1,TotalFreq(TS));
CopyNode( &Node(TS, FreqToIndex(t,TS)), node, VectorSize(TS) );
}


/*-------------------------------------------------------------------*/



void GenerateRandomCodebook(CODEBOOK* CB, TRAININGSET* TS)
{
int i,j;
YESNO Ok;

for (i=0; i<Value(CodebookSize); i++)
    {
    do
        {
        Ok = YES;
        GetRandomVector(&Node(CB,i), TS);
        if (!Value(AllowDuplicates))
	    for (j=0; j<i; j++)
		if (EqualVectors(Vector(CB,i), Vector(CB,j), VectorSize(CB)))
		    Ok=NO;
        }
    while (Ok==NO);    
    }
}


/*-------------------------------------------------------------------*/


void GenerateFullyRandomCodebook(CODEBOOK* CB)

{
int     i,j;
YESNO   Ok;

for (i=0; i<Value(CodebookSize); i++)
    {
    do
        {
        Ok=YES;
        for (j=0; j<VectorSize(CB); j++)
            VectorScalar(CB, i, j) = irand(0,CB->MaxValue);
        VectorFreq(CB, i) = 1;
	if (!Value(AllowDuplicates))
	    for (j=0; j<i; j++)
		if (EqualVectors(Vector(CB,i), Vector(CB,j), VectorSize(CB)))
		    Ok=NO;
        }
    while (Ok==NO);
    }

}


/* ===========================  T I M E  =========================== */

double SetClock(double* start)
{
  struct timeval tv;
  gettimeofday(&tv, NULL);

  return( *start = (double)tv.tv_sec + ((double)tv.tv_usec /1e6) );
}  /* SetClock() */

/* ------------------------------------------------------------------ */

double GetClock(double start)
{
  struct timeval tv;
  double elapsed;

  gettimeofday(&tv, NULL);
  elapsed = (double)tv.tv_sec + ((double)tv.tv_usec /1e6) - start;

  return elapsed;
}  /* GetClock() */


/*===========================  M A I N  ==============================*/

int main(int argc, char* argv[])
{
  TRAININGSET   TS;
  CODEBOOK      CB, CBbest; 
  double        watch;
  double        currError, bestError;
  int           i;
  char          TSName[80];
  char          CBName[80];
  
  
  ParameterInfo paraminfo[2] =  { { TSName,  FormatNameTS, 0, INFILE   }, 
                                  { CBName,  FormatNameCB, 1, OUTFILE  } };
                               
  ParseParameters( argc, argv, 2, paraminfo);

  SetClock(&watch);
  initrandom(0);

  ReadTrainingSet(TSName, &TS);
  CreateNewCodebook(&CB, Value(CodebookSize), &TS);
  CreateNewCodebook(&CBbest, Value(CodebookSize), &TS);


  if ( Value(UseWholeSpace) )
    GenerateFullyRandomCodebook(&CB);
  else
    GenerateRandomCodebook(&CB, &TS);

  CopyCodebook(&CB, &CBbest);
  bestError = AverageErrorCB(&TS, &CBbest, MSE);

  if ( Value(ShowProgress) >= 2 )
    printf(" %d\t%f\t%f\n", 1, GetClock(watch), bestError);

  for (i = 2; i <= Value(CodebookCount); i++) {
    if ( Value(UseWholeSpace) )
      GenerateFullyRandomCodebook(&CB);
    else
      GenerateRandomCodebook(&CB, &TS);
    
    currError = AverageErrorCB(&TS, &CB, MSE);

    if (currError < bestError) {
      CopyCodebook(&CB, &CBbest);
      bestError = currError;
      if ( Value(ShowProgress) >= 2 ) {
        printf(" %d\t%f\t%f", i, GetClock(watch), bestError);
        if ( Value(ShowProgress) >= 3 ) {
          printf("\t(*)");
        }
        printf("\n");
      }
    } else {
      if ( Value(ShowProgress) >= 3 )
        printf(" %d\t%f\t%f\n", i, GetClock(watch), currError);
    }
  }

  if ( Value(ShowProgress) >= 1 ) {
    printf("Time:\t%f\n", GetClock(watch));
    printf("Error:\t%f\n", bestError);
  }

  WriteCodebook(CBName, &CBbest, Value(OverWrite));

  FreeCodebook(&TS);
  FreeCodebook(&CB);
  FreeCodebook(&CBbest);

  return 0;
}


